%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data Analysis Code
% Manuscript titled: "Collective turns in jackdaw flocks: kinematics and information transfer"
% Authors: Hangjian Ling, Guillam E. Mclvor, Joseph Westley, Kasper van der Vaart, Jennify Yin
%          Richard T. Vaughan, Alex Thornton, Nicholas T. Ouellette
% Manuscript submited to "Journal of The Royal Society Interface"
% Date 2019/06
% writen by: Hangjian Ling 
%            Stanford University
% email:     linghj@stanford.edu
%% Figure 2: selected temporal variations of group speed, turn rate, radius, acceleration, polarization, and angular momentum
%            and effect of turn angle on them
%% Figure S11: group size v.s flight speed 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc
%%%% calculate group properties
% for i=1:8
%     load(['turns_mobbing/',sprintf('group_%02d.mat',i)]);
%     [t,Ang,TR,U,An,R,p,m]=GetGroupProp(tracks);
%     Tmiddle=find(TR>0.1*max(TR));
%     TurnTime=t(max(Tmiddle))-t(min(Tmiddle));
%     TurnDeg=Ang(max(Tmiddle))-Ang(min(Tmiddle));
%     BirdIDs=unique(tracks(:,1));
%     GroupN=length(BirdIDs);
%     save(['turns_mobbing/',sprintf('group_ave_%02d.mat',i)],'t','Ang','TR','U','An','R','p','m','TurnDeg','GroupN','TurnTime')
% end
% for i=1:13
%     load(['turns_transit/',sprintf('group_%02d.mat',i)]);
%     [t,Ang,TR,U,An,R,p,m]=GetGroupProp(tracks);
%     Tmiddle=find(TR>0.1*max(TR));
%     TurnTime=t(max(Tmiddle))-t(min(Tmiddle));
%     TurnDeg=Ang(max(Tmiddle))-Ang(min(Tmiddle));
%     BirdIDs=unique(tracks(:,1));
%     GroupN=length(BirdIDs);
%     save(['turns_transit/',sprintf('group_ave_%02d.mat',i)],'t','Ang','TR','U','An','R','p','m','TurnDeg','GroupN','TurnTime')
% end
%%%% collect data
for i=1:8
    load(['turns_mobbing/',sprintf('group_ave_%02d.mat',i)]);
    TurnDeg_all(i)=TurnDeg; % total turn angle
    MinU(i)=min(U); % minimal speed
    MaxTR(i)=max(TR); % maximal turn rate
    MinR(i)=min(R); % min turn radius 
    MinP(i)=min(p);% minimal speed
    MaxAn(i)=max(An); % minimal radial acceleration
    MaxM(i)=max(m); % max angular momentum
    GroupN_all(i)=GroupN; % group size
    TurnTime_all(i)=TurnTime; % time for complete turn
    MeanSpeed(i)=mean(U); % mean speed
end
for i=9:21
    load(['turns_transit/',sprintf('group_ave_%02d.mat',i-8)]);
    TurnDeg_all(i)=TurnDeg; % total turn angle
    MinU(i)=min(U); % minimal speed
    MaxTR(i)=max(TR); % maximal turn rate
    MinR(i)=min(R); % min turn radius 
    MinP(i)=min(p);% minimal speed
    MaxAn(i)=max(An); % minimal radial acceleration
    MaxM(i)=max(m); % max angular momentum
    GroupN_all(i)=GroupN; % group size
    TurnTime_all(i)=TurnTime; % time for complete turn
    MeanSpeed(i)=mean(U); % mean speed
end

%% Figure 2(a): temporal variations of turn rate
figure('units','inches','position',[6 2 3 2.5]); 
load(['turns_transit/',sprintf('group_ave_%02d.mat',6)]);TurnDeg
h1=plot(t/TurnTime,TR,'--','color',[1 0.4 0.4],'LineWidth',3);hold on
load(['turns_transit/',sprintf('group_ave_%02d.mat',12)]);TurnDeg
h2=plot(t/TurnTime,TR,'-','color',[0.4 0.4 1],'LineWidth',3);
axis([-0.55 0.55 0 3]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','on','YMinorTick','on'); 
legend([h1 h2],{'#T06, \theta_g=1.7 rad','#T12, \theta_g=3.8 rad',},'FontSize',11,'FontName','times');%,'Orientation','horizontal'

%% Figure 2(b): effect of turn angle on turn rate
figure('units','inches','position',[2 2 3 2.5]); 
plot(TurnDeg_all(1:8),MaxTR(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
plot(TurnDeg_all(9:21),MaxTR(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
p=polyfit(TurnDeg_all,MaxTR,1); % linear fit
x_fit=[0 5];
plot(x_fit,polyval(p,x_fit),'k--','lineWidth',1.5);
axis([1 5 0 3]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','on','YMinorTick','on');    

%% Figure 2(c): temporal variations of group speed
figure('units','inches','position',[6 2 3 2.5]); 
load(['group_winter/',sprintf('group_ave_%02d.mat',12)]);
plot(t/TurnTime,U,'-','color',[0.4 0.4 1],'LineWidth',3);hold on
load(['group_winter/',sprintf('group_ave_%02d.mat',6)]);
plot(t/TurnTime,U,'--','color',[1 0.4 0.4],'LineWidth',3);
axis([-0.55 0.55 0 13]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 5 10],...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure 2(d): effect of turn angle on group speed
figure('units','inches','position',[2 2 3 2.5]); 
h2=plot(TurnDeg_all(1:8),MinU(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
h1=plot(TurnDeg_all(9:21),MinU(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
p=polyfit(1./TurnDeg_all,MinU,1); % linear fit
x_fit=0.1:0.1:5;
plot(x_fit,polyval(p,1./x_fit),'k--','lineWidth',1.5);
axis([1 5 0 12]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 5 10],...
    'XMinorTick','on','YMinorTick','on'); 
legend([h1 h2],{'transit flocks','mobbing flocks',},'FontSize',12,'FontName','times')

%% Figure 2(e): temporal variations of turn radius
figure('units','inches','position',[6 2 3 2.5]); 
load(['group_winter/',sprintf('group_ave_%02d.mat',12)]);
plot(t/TurnTime,R,'-','color',[0.4 0.4 1],'LineWidth',3);hold on
load(['group_winter/',sprintf('group_ave_%02d.mat',6)]);
plot(t/TurnTime,R,'--','color',[1 0.4 0.4],'LineWidth',3);
axis([-0.55 0.55 1 300]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[1 10 100],'yscale','log',...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure 2(f): effect of turn angle on turn radius
figure('units','inches','position',[2 2 3 2.5]); 
plot(TurnDeg_all(1:8),MinR(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
plot(TurnDeg_all(9:21),MinR(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
p=polyfit((1./TurnDeg_all).^2,MinR,1); % linear fit
x_fit=0.1:0.1:5;
plot(x_fit,polyval(p,(1./x_fit).^2),'k--','lineWidth',1.5);
axis([1 5 0 15]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 5 10 15],...
    'XMinorTick','on','YMinorTick','on');    

%% Figure 2(g): temporal variations of radial acceleration
% figure('units','inches','position',[6 2 3 2.5]); 
% load(['group_winter/',sprintf('group_ave_%02d.mat',12)]);
% plot(t/TurnTime,An,'-','color',[0.4 0.4 1],'LineWidth',3);hold on
% load(['group_winter/',sprintf('group_ave_%02d.mat',6)]);
% plot(t/TurnTime,An,'--','color',[1 0.4 0.4],'LineWidth',3);
% axis([-0.55 0.55 0 10]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 4 8],...
%     'XMinorTick','on','YMinorTick','on'); 

%% Figure 2(h): effect of turn angle on radial acceleration
% figure('units','inches','position',[2 2 3 2.5]); 
% plot(TurnDeg_all(1:8),MaxAn(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
% plot(TurnDeg_all(9:21),MaxAn(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
% plot([1 5],[mean(MaxAn) mean(MaxAn)],'k--','lineWidth',1.5);
% axis([1 5 0 12]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 5 10 15],...
%     'XMinorTick','on','YMinorTick','on'); 

%% Figure 2(i): temporal variations of group polarization
% figure('units','inches','position',[6 2 3 2.5]); 
% load(['group_winter/',sprintf('group_ave_%02d.mat',12)]);
% plot(t/TurnTime,p,'-','color',[0.4 0.4 1],'LineWidth',3);hold on
% load(['group_winter/',sprintf('group_ave_%02d.mat',6)]);
% plot(t/TurnTime,p,'--','color',[1 0.4 0.4],'LineWidth',3);
% axis([-0.55 0.55 0.8 1]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0.8 0.9 1],...
%     'XMinorTick','on','YMinorTick','on'); 

%% Figure 2(j): effect of turn angle on group polarization
% figure('units','inches','position',[2 2 3 2.5]); 
% plot(TurnDeg_all(1:8),MinP(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
% plot(TurnDeg_all(9:21),MinP(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
% p=polyfit(TurnDeg_all,MinP,1); % linear fit
% x_fit=0.1:0.1:6;
% plot(x_fit,polyval(p,x_fit),'k--','lineWidth',1.5);
% axis([1 5 0.55 1]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0.6 0.8 1],...
%     'XMinorTick','on','YMinorTick','on'); 

%% Figure 2(k): effect of turn angle on angular momentum
% figure('units','inches','position',[2 2 3 2.5]); 
% plot(TurnDeg_all(1:8),MaxM(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
% plot(TurnDeg_all(9:21),MaxM(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
% p=polyfit(TurnDeg_all,MaxM,1); % linear fit
% x_fit=0.1:0.1:2*pi;
% plot(x_fit,polyval(p,x_fit),'k--','lineWidth',1.5);
% axis([1 5 0 1]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 0.5 1],...
%     'XMinorTick','on','YMinorTick','on');  

%% Figure 2(l): effect of turn angle on time for complete the turn
% figure('units','inches','position',[2 2 3 2.5]); 
% plot(TurnDeg_all(1:8),TurnTime_all(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
% plot(TurnDeg_all(9:21),TurnTime_all(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
% plot([0 5],[mean(TurnTime_all) mean(TurnTime_all)],'k--','lineWidth',1.5);
% axis([1 5 1 6]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[1 3 5],...
%     'XMinorTick','on','YMinorTick','on');  
% 

%% Figure S11: group size v.s flight speed 
% figure('units','inches','position',[2 2 3.5 3]); 
% plot(GroupN_all(1:8),MeanSpeed(1:8),'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
% % plot(GroupN_all(9:21),MeanSpeed(9:21),'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
% plot([0 80],[mean(MeanSpeed(1:8)) mean(MeanSpeed(1:8))],'k--','lineWidth',1.5);
% axis([0 80 0 10]);box off;
% set(gca,'Position',[0.15 0.15 0.75 0.75])
% set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
%     'XMinorTick','on','YMinorTick','on');  

function [T,Ang,TR,U_ave,An,R,p,m]=GetGroupProp(tracks)
%%%% get time-dependent group density, order, speed, Ang
T=unique(tracks(:,5));
dt=T(2)-T(1);
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8); 
u_ave=mean(u,1);
u_pre=u_ave/sqrt(sum(u_ave.*u_ave));
Ang(1:length(T))=0;
for i=1:length(T)
    id=find(tracks(:,5)==T(i));
    xyz=tracks(id,2:4); 
    u=tracks(id,6:8); 
    a=tracks(id,9:11); 
    U=sum(u.^2,2).^0.5;
    u_norm=u./U;    
    %%%% speed
    u_ave=mean(u,1);
    U_ave(i)=sqrt(sum(u_ave.*u_ave));
        
    %%%% change of angle
    u_ave_norm=u_ave/U_ave(i);
    if i>=2
        Ang(i)=Ang(i-1)+abs(acos(sum(u_ave_norm.*u_pre))); % *180/pi
        u_pre=u_ave_norm;
    end
    %%%% turn raidal acceleration
    a_ave=mean(a,1);
    An_temp=cross(u_ave,a_ave);
    An(i)=sqrt(sum(An_temp.*An_temp))/U_ave(i); 
    %%%% turn radius
    R(i)=U_ave(i)^2/An(i); 
    %%%% turn rate
    TR(i)=U_ave(i)/R(i);

    %%%% group order
    p(i)=1/size(u,1)*sum((sum(u_norm,1)).^2)^0.5;
    %%%% group angular moment
    r=xyz-mean(xyz,1);r=r./sum(r.^2,2).^0.5;
    m(i)=1/size(u,1)*sum((sum(cross(r,u_norm),1)).^2)^0.5;
%     %%%% group density
%     D_temp=[];
%     for j=1:size(xyz,1)
%         Dist_rank=sum((xyz-xyz(j,:)).^2,2).^0.5;        
%         D_temp=[D_temp;max(Dist_rank)];
%     end
%     D(i)=length(BirdIDs)/mean(D_temp)^3*6/pi;
end
%%%% define t=0 as the maximal turn rate
id=find(TR==max(TR));
T=T-T(id);
Ang=Ang-Ang(id);

end

