%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data Analysis Code
% Manuscript titled: "Collective turns in jackdaw flocks: kinematics and information transfer"
% Authors: Hangjian Ling, Guillam E. Mclvor, Joseph Westley, Kasper van der Vaart, Jennify Yin
%          Richard T. Vaughan, Alex Thornton, Nicholas T. Ouellette
% Manuscript submited to "Journal of The Royal Society Interface"
% Date 2019/06
% writen by: Hangjian Ling 
%            Stanford University
% email:     linghj@stanford.edu
%% Figure 3:  
%         (a-c) three cases (T03, T07, M05) showing where turn start 
%         (d) cal. information transfer direction vs=x(ri>0.8N)-x(ri<0.2N), and
%             the volume of first birds start to turn Vs
clear all;clc
%% Figure 3(a). three cases: #W03, #W07, S05
load('turns_transit/group_03.mat'); % T03, 
% % % load('turns_transit/group_07.mat'); % T07
% % % load('turns_mobbing/group_05.mat'); % M05
T=unique(tracks(:,5));
%%%% move coordinate to along flight direction   
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8);
U=mean(u,1);   
theta_temp=atan(U(2)/U(1));
R = [cos(theta_temp) -sin(theta_temp); sin(theta_temp) cos(theta_temp)];
UR=U(1:2)*R;UR(3)=U(3);
for j=1:size(tracks,1)
    if UR(1)>0
        tracks(j,2:3) = tracks(j,2:3)*R; 
        tracks(j,6:7)=tracks(j,6:7)*R;
    elseif UR(1)<0
        tracks(j,2:3) = tracks(j,2:3)*R*(-1); 
        tracks(j,6:7)=tracks(j,6:7)*R*(-1);  
    end
end
%%%%%%%%%%%%%%%%%% end the movement
N=length(unique(tracks(:,1)));
[Rank,Delay]=GetRank(tracks);
[T,TR]=GetGroupProp(tracks);
Tmiddle=find(TR>0.1*max(TR));
id=find(tracks(:,5)==T(min(Tmiddle))); % time frame when turn start
xyz=tracks(id,2:4);
u=tracks(id,6:8);
u=u./sqrt(sum(u.*u,2));
ID1=find(Rank<0.2*N);
ID2=find(Rank>0.8*N);
ID3=find(Rank<=0.8*N & Rank>=0.2*N);
figure('units','inches','position',[2 2 3.5 3]); 
scatter(xyz(ID3,1),xyz(ID3,2),100,[0.7 0.7 0.7],'o','filled',...
    'LineWidth',1,'MarkerEdgeColor',[0.3 0.3 0.3]);hold on
scatter(xyz(ID1,1),xyz(ID1,2),100,[1 0.3 0.3],'o','filled',...
    'LineWidth',1,'MarkerEdgeColor','k');hold on
scatter(xyz(ID2,1),xyz(ID2,2),100,[0.3 0.3 1],'o','filled',...
    'LineWidth',1,'MarkerEdgeColor','k');hold on
quiver(xyz(:,1),xyz(:,2),u(:,1),u(:,2),...
    'color',[0.2 0.2 0.2],'linewidth',1,'AutoScaleFactor',0.5)
grid off;axis off

%% figure 3(d): for all the cases, get angle between vs to ug 
% for i=1:13
%     load(['turns_transit/',sprintf('group_%02d.mat',i)]);
%     [Rank,Delay]=GetRank(tracks); 
%     T=unique(tracks(:,5));
%     N=length(unique(tracks(:,1)));
%     [T,TR]=GetGroupProp(tracks);
%     Tmiddle=find(TR>0.1*max(TR));
%     id=find(tracks(:,5)==T(min(Tmiddle))); % time frame when turn start
%     xyz=tracks(id,2:4);
%     u=tracks(id,6:8);
%     ug=mean(u,1);
%     Ug(i)=sum(ug.^2)^0.5;
%     ug=ug/sum(ug.^2)^0.5;
%     ID1=find(Rank<0.2*N);
%     ID2=find(Rank>0.8*N);
%     vs=mean(xyz(ID1,:),1)-mean(xyz(ID2,:),1); % opposit to propagate direction
%     vs=vs/sum(vs.^2)^0.5;
%     %%% get angle 
%     Ang(i)=acos(sum(ug.*vs));
%     %%% get volume of entire group xyz(ID1,:)
%     D_temp_all=[];
%     for j=1:size(xyz,1)
%         Dist_rank=sum((xyz-xyz(j,:)).^2,2).^0.5;        
%         D_temp_all=[D_temp_all;max(Dist_rank)];
%     end
%     D_all=mean(D_temp_all);
%     %%% get volume of entire group xyz(ID1,:)
%     xyz_sub=xyz(ID1,:);
%     D_temp_sub=[];
%     for j=1:size(xyz_sub,1)
%         Dist_rank=sum((xyz_sub-xyz_sub(j,:)).^2,2).^0.5;        
%         D_temp_sub=[D_temp_sub;max(Dist_rank)];
%     end
%     D_sub=mean(D_temp_sub);
%     %%%% get volume ratio
%     Vsub2all(i)=(D_sub/D_all)^3;
%     %%%% get group order
%     p=1/size(u,1)*sum((sum(u./sum(u.^2,2).^0.5,1)).^2)^0.5;
%     p_all(i)=p;
%     N_all(i)=N;
% end
% save('Info_turnStart_T.mat','Vsub2all','Ang','N_all','p_all','Ug');

%%%%% plot the results
figure('units','inches','position',[2 2 3 2.5]);
load('Info_turnStart_T.mat');
h1=plot(Ang*180/pi,Vsub2all,'ko','MarkerSize',10,'MarkerFaceColor',[0.3 1 0.3]);hold on
load('Info_turnStart_M.mat');
h2=plot(Ang*180/pi,Vsub2all,'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
axis([0 240 0 1]);
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','off','YMinorTick','off');

function [T,TR]=GetGroupProp(tracks)
%%%% get time-dependent group density, order, speed, Ang
T=unique(tracks(:,5));
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8); 
u_ave=mean(u,1);
u_pre=u_ave/sqrt(sum(u_ave.*u_ave));
Ang(1:length(T))=0;
for i=1:length(T)
    id=find(tracks(:,5)==T(i));
    xyz=tracks(id,2:4); 
    u=tracks(id,6:8); 
    a=tracks(id,9:11); 
    U=sum(u.^2,2).^0.5;
    u_norm=u./U;    
    %%%% speed
    u_ave=mean(u,1);
    U_ave(i)=sqrt(sum(u_ave.*u_ave));
    %%%% change of angle
    u_ave_norm=u_ave/U_ave(i);
    if i>=2
        Ang(i)=Ang(i-1)+abs(acos(sum(u_ave_norm.*u_pre))); % *180/pi
        u_pre=u_ave_norm;
    end
    %%%% turn raidal acceleration
    a_ave=mean(a,1);
    An_temp=cross(u_ave,a_ave);
    An(i)=sqrt(sum(An_temp.*An_temp))/U_ave(i); 
    %%%% turn radius
    R(i)=U_ave(i)^2/An(i); 
    %%%% turn rate
    TR(i)=U_ave(i)/R(i);
end
end


