%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data Analysis Code
% Manuscript titled: "Collective turns in jackdaw flocks: kinematics and information transfer"
% Authors: Hangjian Ling, Guillam E. Mclvor, Joseph Westley, Kasper van der Vaart, Jennify Yin
%          Richard T. Vaughan, Alex Thornton, Nicholas T. Ouellette
% Manuscript submited to "Journal of The Royal Society Interface"
% Date 2019/06
% writen by: Hangjian Ling 
%            Stanford University
% email:     linghj@stanford.edu
%% Figure 4: information propagate speed Cs
%        (a) plot sample cases showing information propagate through the flock
%        (b) plot sample profiles of di(ti) for the same case
%        (c) plot group size effect on Cs
%        (d) plot group polarization effect on Cs
clear all;clc
load('/Users/ling/Downloads/ScientificColourMaps5/roma/roma.mat');
%% 4(a) plot bird distribution color by ti for mobbing flock #M05
load('turns_mobbing/group_05.mat'); 
load(['turns_mobbing/','group_delay_05.mat']);
T=unique(tracks(:,5));
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8);
U=mean(u,1);   
theta_temp=atan(U(2)/U(1));
R = [cos(theta_temp) -sin(theta_temp); sin(theta_temp) cos(theta_temp)];
UR=U(1:2)*R;UR(3)=U(3);
for j=1:size(tracks,1)
    if UR(1)>0
        tracks(j,2:3) = tracks(j,2:3)*R; 
        tracks(j,6:7)=tracks(j,6:7)*R;
    elseif UR(1)<0
        tracks(j,2:3) = tracks(j,2:3)*R*(-1); 
        tracks(j,6:7)=tracks(j,6:7)*R*(-1);  
    end
end
[T,TR]=GetGroupProp(tracks);
Tmiddle=find(TR>0.1*max(TR));
id=find(tracks(:,5)==T(min(Tmiddle))); % time frame when turn start
xyz=tracks(id,2:4);
u=tracks(id,6:8);
u=u./sqrt(sum(u.*u,2));
figure('units','inches','position',[2 2 3 2.5]); 
scatter(xyz(:,1),xyz(:,2),80,Delay,'o','filled',...
    'LineWidth',1,'MarkerEdgeColor',[0.3 0.3 0.3]);hold on
quiver(xyz(:,1),xyz(:,2),u(:,1),u(:,2),...
    'color',[0.2 0.2 0.2],'linewidth',1,'AutoScaleFactor',0.5)
colormap(roma);box off
colorbar('ticks',[],'FontSize',10,...
    'Location','north','Position',[0.45 0.82 0.4 0.04]);
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','on','YMinorTick','on');

%% 4(b) Plot profile of di(ti) for mobbing flock #M05
load(['turns_mobbing/','group_delay_05.mat']);
figure('units','inches','position',[5 2 3 2.5]); 
Delay=sort(Delay);     
x_fit=0:0.1:4;
plot(Delay(1:2:end),InfoDis(1:2:end),'kd','MarkerFaceColor',[0.3 0.3 0.8],'MarkerSize',9);hold on 
p=polyfit(Delay(5:end-15),InfoDis(5:end-15),1);
h1=plot(x_fit,polyval(p,x_fit),'k-','LineWidth',1.5); 
axis([0 3. 0 30]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','on','YMinorTick','on'); % 'xscale','log','yscale','log'

%% 4(c-d) Plot group size, polarization effects on Cs for mobbing flocks
Order=[0.88 0.83 0.79 0.87 0.84 0.92 0.88 0.94];
N=[8 4 44 57 70 5 34 4];
Cs_H=[4.9 4.1 7.4 6.5 7.4 4.2 8.0 3.9];
Cs_L=[3.9 2.4 5.2 5.3 6.2 2.3 7.0 1.5];
Cs=(Cs_H+Cs_L)/2;
Err=(Cs_H-Cs_L)/2;

figure('units','inches','position',[2 2 3 2.5]); 
errorbar(N,Cs,Err,'k','LineWidth',1.5,'LineStyle','none');hold on
h1=plot(N,Cs,'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
x_fit=1:80;
y_fit=6.6-6.5*exp(-1*x_fit/7.1);
h2=plot(x_fit,y_fit,'r-','LineWidth',1.5);
axis([0 80 0 9]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'xscale','linear','yscale','linear','XMinorTick','on','YMinorTick','on'); 

figure('units','inches','position',[5 2 3 2.5]); 
errorbar(Order,Cs,Err,'k','LineWidth',1.5,'LineStyle','none');hold on
h1=plot(Order,Cs,'k^','MarkerSize',10,'MarkerFaceColor',[1 0.3 0.3]);hold on
axis([0.75 1 0 9]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'xscale','linear','yscale','linear','XMinorTick','on','YMinorTick','on'); 


function [T,TR]=GetGroupProp(tracks)
%%%% get time-dependent group density, order, speed, Ang
T=unique(tracks(:,5));
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8); 
u_ave=mean(u,1);
u_pre=u_ave/sqrt(sum(u_ave.*u_ave));
Ang(1:length(T))=0;
for i=1:length(T)
    id=find(tracks(:,5)==T(i));
    xyz=tracks(id,2:4); 
    u=tracks(id,6:8); 
    a=tracks(id,9:11); 
    U=sum(u.^2,2).^0.5;
    u_norm=u./U;    
    %%%% speed
    u_ave=mean(u,1);
    U_ave(i)=sqrt(sum(u_ave.*u_ave));
    %%%% change of angle
    u_ave_norm=u_ave/U_ave(i);
    if i>=2
        Ang(i)=Ang(i-1)+abs(acos(sum(u_ave_norm.*u_pre))); % *180/pi
        u_pre=u_ave_norm;
    end
    %%%% turn raidal acceleration
    a_ave=mean(a,1);
    An_temp=cross(u_ave,a_ave);
    An(i)=sqrt(sum(An_temp.*An_temp))/U_ave(i); 
    %%%% turn radius
    R(i)=U_ave(i)^2/An(i); 
    %%%% turn rate
    TR(i)=U_ave(i)/R(i);
end
end