%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data Analysis Code
% Manuscript titled: "Collective turns in jackdaw flocks: kinematics and information transfer"
% Authors: Hangjian Ling, Guillam E. Mclvor, Joseph Westley, Kasper van der Vaart, Jennify Yin
%          Richard T. Vaughan, Alex Thornton, Nicholas T. Ouellette
% Manuscript submited to "Journal of The Royal Society Interface"
% Date 2019/06
% writen by: Hangjian Ling 
%            Stanford University
% email:     linghj@stanford.edu
%% Figure S12: subgroups in transit flock during turns
clear all;clc
%%%% transit flock #T03 at Frame=1;130;260
load('turns_transit/group_03.mat');Frame=260;

%%%% mobbing flock #M05 at Frame=1;130;260
% load('turns_mobbing/group_05.mat');Frame=260; 

T=unique(tracks(:,5));
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8);
U=mean(u,1);   
theta_temp=atan(U(2)/U(1));
R = [cos(theta_temp) -sin(theta_temp); sin(theta_temp) cos(theta_temp)];
UR=U(1:2)*R;UR(3)=U(3);
for j=1:size(tracks,1)
    if UR(1)>0
        tracks(j,2:3) = tracks(j,2:3)*R; 
        tracks(j,6:7)=tracks(j,6:7)*R;
    elseif UR(1)<0
        tracks(j,2:3) = tracks(j,2:3)*R*(-1); 
        tracks(j,6:7)=tracks(j,6:7)*R*(-1);  
    end
end
[Rank,Delay]=GetRank(tracks);
Bound=[min(tracks(:,2))-5 max(tracks(:,2)+5) ...
    min(tracks(:,3))-5 max(tracks(:,3)+5)];

id=find(tracks(:,5)==T(Frame));
NumT=tracks(id,1);
xyz=tracks(id,2:4); 
u=tracks(id,6:8);
U=sum(u.^2,2).^0.5;
u=u./U;

figure('units','inches','position',[2 2 3 2.5]);   
scatter(xyz(:,1),xyz(:,2),40,[0.6 0.6 0.6],'o','filled','MarkerEdgeColor','k');hold on
quiver(xyz(:,1),xyz(:,2),u(:,1),u(:,2),...
    'color',[0.6 0.2 0.2],'linewidth',1,'AutoScaleFactor',0.5)
box on;axis(Bound);
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01]);

