%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data Analysis Code
% Manuscript titled: "Collective turns in jackdaw flocks: kinematics and information transfer"
% Authors: Hangjian Ling, Guillam E. Mclvor, Joseph Westley, Kasper van der Vaart, Jennify Yin
%          Richard T. Vaughan, Alex Thornton, Nicholas T. Ouellette
% Manuscript submited to "Journal of The Royal Society Interface"
% Date 2019/06
% writen by: Hangjian Ling 
%            Stanford University
% email:     linghj@stanford.edu
%% Figure S1: plot a turn made by a group (example transit flock #04) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc
load('/Users/ling/Downloads/ScientificColourMaps5/roma/roma.mat');
load('turns_transit/group_04.mat'); % transit flock #04
BirdIDs=unique(tracks(:,1));
T=unique(tracks(:,5));
[Rank,Delay]=GetRank(tracks);
%%%% move coordinate to along flight direction   
id=find(tracks(:,5)==T(1));
u=tracks(id,6:8);
U=mean(u,1);   
theta_temp=atan(U(2)/U(1));
R = [cos(theta_temp) -sin(theta_temp); sin(theta_temp) cos(theta_temp)];
UR=U(1:2)*R;UR(3)=U(3);
for j=1:size(tracks,1)
    if UR(1)>0
        tracks(j,2:3) = tracks(j,2:3)*R; 
        tracks(j,6:7)=tracks(j,6:7)*R;
    elseif UR(1)<0
        tracks(j,2:3) = tracks(j,2:3)*R*(-1); 
        tracks(j,6:7)=tracks(j,6:7)*R*(-1);  
    end
end
%%%% boundary limit of the imaging volume
BD=[min(tracks(:,2))-1 max(tracks(:,2))+1 ...
    min(tracks(:,3))-1 max(tracks(:,3))+1 ...
    min(tracks(:,4))-1 max(tracks(:,4))+1];

%% Figure 1(a): bird positions colored by time lag, in 3D, at different time steps
figure('units','inches','position',[2 2 12 10]); 
id=find(tracks(:,5)==T(1)); %% T(1), T(60), T(120), T(180), T(240), T(300)
xyz=tracks(id,2:4);
u=tracks(id,6:8);
u=u./sqrt(sum(u.*u,2));
scatter3(xyz(:,1),xyz(:,2),xyz(:,3),300,Delay,'o','filled',...
    'LineWidth',1.5,'MarkerEdgeColor','k');hold on
quiver3(xyz(:,1),xyz(:,2),xyz(:,3),u(:,1),u(:,2),u(:,3),...
    'color',[0.2 0.2 0.2],'linewidth',3,'AutoScaleFactor',0.5)
axis(BD);colormap(roma);grid off;box on
% colorbar('ticks',[],'FontSize',30,...
%     'Location','north','Position',[0.15 0.92 0.7 0.04]);
set(gca,'FontSize',36,'FontName','times');   
set(gca,'linewidth',3)

%% Figure 1(b): 3 sample bird trajectory colored, in 3D
figure('units','inches','position',[2 2 12 10]); 
id1=find(Delay==min(Delay));
id1=find(tracks(:,1)==BirdIDs(id1(1)));
xyz=tracks(id1,2:4);
h1=plot3(xyz(:,1),xyz(:,2),xyz(:,3),'-','color',[0.8 0.3 0.3],'LineWidth',6);hold on
scatter3(xyz(end,1),xyz(end,2),xyz(end,3),500,[0.8 0.3 0.3],'o','filled',...
    'LineWidth',3,'MarkerEdgeColor','k');hold on

id1=find(abs(Delay-max(Delay)/2)==min(abs(Delay-max(Delay)/2))); % mid bird turn
id1=find(tracks(:,1)==BirdIDs(id1(1)));
xyz=tracks(id1,2:4);
h2=plot3(xyz(:,1),xyz(:,2),xyz(:,3),'-','color',[0.3 0.8 0.3],'LineWidth',6);hold on
scatter3(xyz(end,1),xyz(end,2),xyz(end,3),500,[0.3 0.8 0.3],'o','filled',...
    'LineWidth',3,'MarkerEdgeColor','k');hold on

id1=find(Delay==max(Delay));
id1=find(tracks(:,1)==BirdIDs(id1(1)));
xyz=tracks(id1,2:4);
h3=plot3(xyz(:,1),xyz(:,2),xyz(:,3),'-','color',[0.3 0.3 0.8],'LineWidth',6);hold on
scatter3(xyz(end,1),xyz(end,2),xyz(end,3),500,[0.3 0.3 0.8],'o','filled',...
    'LineWidth',3,'MarkerEdgeColor','k');hold on

axis(BD);colormap(roma);grid off;box on
set(gca,'FontSize',36,'FontName','times');   
set(gca,'linewidth',3)

%% Figure 1(c): 3 sample bird flight speed
figure('units','inches','position',[2 2 3.5 3]); 
id1=find(Delay==min(Delay));
id1=find(tracks(:,1)==BirdIDs(id1(1))); % first bird turn
u=tracks(id1,6:8);U=sum(u.^2,2).^0.5;
h1=plot(T-min(T),U,'-','color',[0.8 0.3 0.3],'LineWidth',3);hold on

id1=find(abs(Delay-max(Delay)/2)==min(abs(Delay-max(Delay)/2))); % mid bird turn
id1=find(tracks(:,1)==BirdIDs(id1(1))); % mid bird turn
u=tracks(id1,6:8);U=sum(u.^2,2).^0.5;
h2=plot(T-min(T),U,'-','color',[0.3 0.8 0.3],'LineWidth',3);hold on

id1=find(Delay==max(Delay));
id1=find(tracks(:,1)==BirdIDs(id1(1))); % last bird turn
u=tracks(id1,6:8);U=sum(u.^2,2).^0.5;
h3=plot(T-min(T),U,'-','color',[0.3 0.3 0.8],'LineWidth',3);hold on

set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','on','YMinorTick','on'); 

