%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data Analysis Code
% Manuscript titled: "Collective turns in jackdaw flocks: kinematics and information transfer"
% Authors: Hangjian Ling, Guillam E. Mclvor, Joseph Westley, Kasper van der Vaart, Jennify Yin
%          Richard T. Vaughan, Alex Thornton, Nicholas T. Ouellette
% Manuscript submited to "Journal of The Royal Society Interface"
% Date 2019/06
% writen by: Hangjian Ling 
%            Stanford University
% email:     linghj@stanford.edu
%% Figure S4,S5: temporal variations of group speed, turn rate, radius, acceleration, polarization, and angular momentum
%            for all flocks
clear all;clc
load('/Users/ling/Downloads/ScientificColourMaps5/vik/vik.mat');

%% Figure S4,5(a): temporal variations of turn rate
figure('units','inches','position',[6 2 3.5 3]); 
for i=1:13
    load(['group_winter/',sprintf('group_ave_%02d.mat',i)]);
    scatter(t/TurnTime,TR,20,repmat(TurnDeg,1,length(t)),'o','filled');hold on
    plot(t/TurnTime,TR,'-','color',[0.4 0.4 0.4]);hold on;
end
colormap(vik);
axis([-0.55 0.55 0 4]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 1 2 3],...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure S4,5(b): temporal variations of group speed
figure('units','inches','position',[6 2 3.5 3]); 
for i=1:13
    load(['group_winter/',sprintf('group_ave_%02d.mat',i)]);
    scatter(t/TurnTime,U,20,repmat(TurnDeg,1,length(t)),'o','filled');hold on
    plot(t/TurnTime,U,'-','color',[0.4 0.4 0.4]);hold on;
end
colormap(vik);
colorbar('ticks',[],'FontSize',12,...
    'Location','north','Position',[0.3 0.85 0.5 0.04]);
axis([-0.55 0.55 0 13]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 5 10],...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure S4,5(c): temporal variations of turn radius
figure('units','inches','position',[6 2 3.5 3]); 
for i=1:13
    load(['group_winter/',sprintf('group_ave_%02d.mat',i)]);
    scatter(t/TurnTime,R,20,repmat(TurnDeg,1,length(t)),'o','filled');hold on
    plot(t/TurnTime,R,'-','color',[0.4 0.4 0.4]);hold on;
end
colormap(vik);
axis([-0.55 0.55 0 2000]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'yscale','log','ytick',[1 10 100 1000],...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure S4,5(d): temporal variations of radial acceleration
figure('units','inches','position',[6 2 3.5 3]); 
for i=1:13
    load(['group_winter/',sprintf('group_ave_%02d.mat',i)]);
    scatter(t/TurnTime,An,20,repmat(TurnDeg,1,length(t)),'o','filled');hold on
    plot(t/TurnTime,An,'-','color',[0.4 0.4 0.4]);hold on;
end
colormap(vik);
axis([-0.55 0.55 0 10]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 5 10],...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure S4,5(e): temporal variations of polarization
figure('units','inches','position',[6 2 3.5 3]); 
for i=1:13
    load(['group_winter/',sprintf('group_ave_%02d.mat',i)]);
    scatter(t/TurnTime,p,20,repmat(TurnDeg,1,length(t)),'o','filled');hold on
    plot(t/TurnTime,p,'-','color',[0.4 0.4 0.4]);hold on;
end
colormap(vik);
axis([-0.55 0.55 0 1.05]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],...
    'XMinorTick','on','YMinorTick','on'); 

%% Figure S4,5(f): temporal variations of angular momentum
figure('units','inches','position',[6 2 3.5 3]); 
for i=1:13
    load(['group_winter/',sprintf('group_ave_%02d.mat',i)]);
    scatter(t/TurnTime,m,20,repmat(TurnDeg,1,length(t)),'o','filled');hold on
    plot(t/TurnTime,m,'-','color',[0.4 0.4 0.4]);hold on;
end
colormap(vik);
axis([-0.55 0.55 0 1]);box off;
set(gca,'Position',[0.15 0.15 0.75 0.75])
set(gca,'FontSize',12,'FontName','times','TickLength',[0.03, 0.01],'ytick',[0 0.5 1],...
    'XMinorTick','on','YMinorTick','on'); 